"""Exports class Actor, all characters in the game are of class Actor."""

from osm import collective
import movement
import animation

class Actor(collective.Collective):
    """A class for characters in the game. It has members: movement, visual and animPlayer.
    Movement updates actor's position. Visual provides rendering functions, allowing to 
    display selected image. The image and time to display is selected by animPlayer."""
    def __init__(self, name, script):
        collective.Collective.__init__(self, name, script)
        self.visual = None
        self.movement = movement.Movement(name=self.name+"::movement")
        self.animPlayer = animation.AnimationPlayer(name=self.name+"::animPlayer")
        self.addMember(self.movement)
        self.addMember(self.animPlayer)
        self.messageHandlers["goto"] = self.Handler_OnGoTo
        self.messageHandlers["init"] = self.Handler_OnInit
        self.messageHandlers["setposition"] = self.Handler_OnSetPosition
        self.actions["walking"] = self.Action_Walking
    
    def setVisual(self,v):
        """Assigns self.visual=v and adds v to the members. 
        Also sends command to animPlayer to show 'idle' image."""
        if (self.visual):
            print "ERROR: ",self.name," ALREADY HAS VISUAL!!! self.visual=", self.visual
        else:
            #print self.name," got visual: ",v
            self.visual = v
            self.visual.name = self.name+"::visual"
            self.addMember(self.visual)
            self.visual.sendCommand("show image",paramsDict={"image":"idle"})
    
    def Handler_OnInit(self,msg):
        """Default initialization code, usually overriden by script."""
        #print "Handler_OnInit"
        self.animPlayer.sendCommand("start",paramsDict={"name":"idle"})
    
    def Handler_OnGoTo(self,msg):
        """Default implementation for the GoTo command handler. 
        Sends appropriate commands to movement and animPlayer."""
        dest = msg.paramByName("destination")
        #print "received goto command, dest=",dest
        self.movement.postCommand("goto",paramsDict={"destination":dest})
        self.animPlayer.sendCommand("start",paramsDict={"name":"walk"})
        self.setState("walking")

    def Action_Walking(self):
        """Default update function (action) for actor movement."""
        for msg in self.movement.outBox:
            if (msg.type=="translation"):
                currPos = msg.paramByName("currPos")
                delta = msg.paramByName("delta")
                newPos = currPos + delta
                if (self.visual):
                    self.visual.postCommand("setposition",paramsDict={"newPos":newPos})
            if (msg.type=="idle"):
                self.animPlayer.sendCommand("start",paramsDict={"name":"idle"})
                self.setState("idle")

    # utility functions 
    def distTo(self, other):
        """Utility function, returns distance to the other object"""
        return abs(self.movement.currPos-other.movement.currPos)

    def dirTo(self, other):
        """Utility function, returns direction to the other object"""
        return other.movement.currPos-self.movement.currPos

    def Handler_OnSetPosition(self,msg):
        self.movement.sendCommand("setposition",paramsDict=msg.dict)
        self.visual.sendCommand("setposition",paramsDict=msg.dict)
        
