"""A Tkinter-based renderer for actors."""

import bitmap
import Tkinter
import ImageTk
import Image
from osm import agent

class ActorVisual(bitmap.Bitmap, agent.Agent):
    """Renders actors by showing images on the Tkiner canvas 
    object. It caches images so that images called once stay in 
    memory for faster access in the future."""
    
    def __init__(self, canvas, imgDict, name):
        agent.Agent.__init__(self,name)
        self.currImg = "idle"
        self.imgDict = imgDict
        imgFile = imgDict[self.currImg]
        bitmap.Bitmap.__init__(self, canvas, imgFile)
        self.shift = self.size[0]+5
        self.textId = self.canvas.create_text(self.pos.real, self.pos.imag+self.shift, text=name)
        self.healthId = None
        self.canvas.pack(fill=Tkinter.BOTH,expand=Tkinter.YES)
        self.cache = {}
        self.messageHandlers["show image"] = self.Handler_OnShowImage
        self.messageHandlers["setposition"] = self.Handler_OnSetPos
        self.messageHandlers["shut down"] = self.Handler_OnShutDown
        self.messageHandlers["set health bar"] = self.Handler_OnSetHealthBar

    def Handler_OnShowImage(self, msg):
        """Shows requested image. If the image is already in cache, it retrieves it
        from cache and displays, otherwise it loads image first and places it in the
        cache."""
        newImage = msg.paramByName("image")
        if (self.currImg != newImage) and (self.imgDict.has_key(newImage)):
            self.canvas.delete(self.id)
            imgFile = self.imgDict[newImage]
            if (self.cache.has_key(newImage)):
                self.picture = self.cache[newImage]
            else:
                img = Image.open(imgFile)
                self.picture = ImageTk.PhotoImage(img)
                self.cache[newImage] = self.picture
            self.id = self.canvas.create_image(self.pos.real, self.pos.imag, anchor=Tkinter.NW, image = self.picture)
            self.currImg = newImage
    
    def Handler_OnSetPos(self, msg):
        """Places actor's image at the specified location on the canvas by calling updatePos."""
        self.updatePos(msg.paramByName("newPos"))

    def Handler_OnShutDown(self,msg):
        """Cleans the cache of images to facilitate garbage collection. 
        Not called in the current version."""
        self.canvas.delete(self.id)
        self.canvas.delete(self.textId)
        if (self.healthId):
            self.canvas.delete(self.healthId)
            
    def Handler_OnSetHealthBar(self, msg):
        """Displays health bar for the character. Not called in the current version."""
        txt = str(msg.health)
        if (self.healthId):
            self.canvas.delete(self.healthId)
            self.healthId = None
        self.healthId = self.canvas.create_text(self.pos.real+27, self.pos.imag, text=txt)
   
    def updatePos(self, newPos):
        """Actually moves actor's image to the specified location by configuring 
        coordinates of the image, health bar and object label."""
        self.pos = newPos
        self.canvas.coords(self.id, self.pos.real, self.pos.imag)
        self.canvas.coords(self.textId, self.pos.real, self.pos.imag + self.shift)
        if (self.healthId):
            self.canvas.coords(self.healthId, self.pos.real+self.shift, self.pos.imag)

