"""Defines Game class. The Game object is (moslty) independed from
the Tkinter UI. It is derived from Collective and has very few hardcoded
methods, with most of its functinality implemented in game-specific 
scripts."""

from osm import collective
import bitmap
import datapath

class Game(collective.Collective):
    """A top level collective object, it represents the game itself.
    It reports directly to gameui object. The game object is responsible
    for spawning and controlling objects necessary for the game to start 
    and run (groups, dispatchers, etc). It implements only minimal set 
    of functions leaving all the actual game functinality to the game 
    script."""
    
    def __init__(self, root, canvas):
        collective.Collective.__init__(self, name="Game")
        self.canvas = canvas  # game will draw itself on the canvas
        self.root = root      # Tkinter main object
        self.messageHandlers["run"] = self.Handler_Run
        self.frame = 0

    def initBackground(self, filename):
        """Sets the background map for the game field, probably should be 
        implemented in scripts."""
        self.map = bitmap.Bitmap(self.canvas, datapath.fullname(filename))
    
    def Handler_Run(self, msg):
        """Default handler for 'run' command, switches state to 'running'."""
        self.stepsToRun = msg.paramByName("nSteps")
        self.setState("running")
    
    def nextStep(self):
        """Makes the game to advance to the next frame (if stepsToRun>0) 
        and decrements stepsToRun."""
        if (self.stepsToRun==0):
            self.setState("idle")
            print "game is stopped"
        else:
            self.frame += 1
            #print "frame=",self.frame
            #print "game is running ",g.stepsToRun
        self.stepsToRun -= 1
