import Tkinter
import Pmw
import tkFileDialog
import os
import xmlscript
import datapath
import sys

#import Pmw:
#sys.path[:0] = ['D:\\Python23']
sys.path[:0] = ['..\\..\\']
import Pmw

class ScriptEditor:
    """A tool for viewing, browsing (and, soon, editing) scripts."""
    def __init__(self, startPath, root=None):
        self.startPath = startPath
        self.gamefile = None
        self.init()
        self.standalone = (root==None)
        if (root==None):
            self.root=Tkinter.Tk()
        else:
            self.root = root            
        self.buildTopMenu()
        self.buildFileNamesFrame()
        self.buildXmlFilesList()
        self.buildCanvas()
        if (root==None):
            self.root.mainloop()

    def init(self):
        self.xmlFiles = {}
        self.scripts = {}
        self.scriptNames = []
        self.currentScript = None
        self.currentMethod = None
        self.allScripts = None
        self.modifiedScripts = []
        self.modifiedMethods = [] #keeps pairs (script,method)
    
    def resetui(self):
        self.scriptList.clear()
        self.inheritedList.clear()
        self.methodList.clear()
        self.methodText.clear()
        self.resetCanvas()
        
    def resetCanvas(self):
        self.canvas.destroy()
        self.canvas = Pmw.ScrolledCanvas(self.root,borderframe = 1,canvasmargin=10)
        self.canvas.pack(side=Tkinter.TOP, fill=Tkinter.BOTH, expand=Tkinter.YES)
        
    def buildTopMenu(self):
        self.mBar = Tkinter.Frame(self.root, relief=Tkinter.RAISED, borderwidth=2)
        self.mBar.pack(fill=Tkinter.X)
        self.gameBtn = Tkinter.Menubutton(self.mBar,text="Game", underline=0)
        self.gameBtn.pack(side=Tkinter.LEFT, padx="1m")
        self.gameBtn.menu = Tkinter.Menu(self.gameBtn,tearoff=0)
        if (self.standalone):
            self.gameBtn.menu.add_command(label="Open...", underline=0, command = self.openGame)
        self.gameBtn.menu.add_command(label="Save All", underline=0, command = None)
        self.gameBtn.menu.entryconfigure(index=2, state = Tkinter.DISABLED)
        self.gameBtn['menu'] = self.gameBtn.menu

        self.scriptBtn = Tkinter.Menubutton(self.mBar,text="Script", underline=0)
        self.scriptBtn.pack(side=Tkinter.LEFT, padx="1m")
        self.scriptBtn.menu = Tkinter.Menu(self.scriptBtn)
        self.scriptBtn.menu.add_command(label="New...", underline=0, command = self.newScriptDlg)
        self.scriptBtn.menu.add_command(label="Add Inherited...", underline=0, command = self.addInheritedDlg)
        self.scriptBtn.menu.entryconfigure(index=2, state = Tkinter.DISABLED)
        self.scriptBtn.menu.add_command(label="Save", underline=0, command = self.saveCurrScript)
        self.scriptBtn.menu.entryconfigure(index=3, state = Tkinter.DISABLED)
        #~ self.scriptBtn.menu.add_command(label="Save As...", underline=0, command = None)
        #~ self.scriptBtn.menu.add_command(label="Remove", underline=0, command = None)
        #~ self.scriptBtn.menu.add_command(label="Rename...", underline=0, command = None)
        #~ self.scriptBtn.menu.add_command(label="Remove Inheritance From...", underline=0, command = None)
        self.scriptBtn['menu'] = self.scriptBtn.menu
        self.scriptBtn['state'] = Tkinter.DISABLED

        self.methodBtn = Tkinter.Menubutton(self.mBar,text="Method", underline=0)
        self.methodBtn.pack(side=Tkinter.LEFT, padx="1m")
        self.methodBtn.menu = Tkinter.Menu(self.methodBtn)
        self.methodBtn.menu.add_command(label="New...", underline=0, command = self.newMethodDlg)
        self.methodBtn.menu.add_command(label="Save", underline=0, command = self.saveMethod)
        self.methodBtn.menu.entryconfigure(index=2, state = Tkinter.DISABLED)
        #~ self.methodBtn.menu.add_command(label="Save As...", underline=0, command = None)
        #~ self.methodBtn.menu.add_command(label="Remove", underline=0, command = None)
        #~ self.methodBtn.menu.add_command(label="Rename...", underline=0, command = None)
        self.methodBtn['menu'] = self.methodBtn.menu
        self.methodBtn['state'] = Tkinter.DISABLED
    
    def scriptIsModified(self,aScript):
        try:
            self.modifiedScripts.index(aScript)
        except:
            return 0
        return 1
        
    def selectedMethodIsModified(self):
        if (self.currentMethod==None):
            return 0
        if (self.currentScript[self.currentMethod]==self.methodText.get()):
            return 0
        return 1
        
    def updateFileNameLabel(self, selectedScript, selectedMethod=None):
        combinedName = self.xmlFiles[selectedScript.name]
        if (self.scriptIsModified(selectedScript)):
            combinedName += " (*) "
        if (selectedMethod!=None):
            #check that method is not from inherited script, hence has no "."!!! will throw exception
            if (selectedMethod.find(".")>=0):
                combinedName += "  ::  inherited: " + selectedMethod
                #find inherited script, find method file in there!!!
            else:
                combinedName += "  ::  " + self.currentScript.fileNames[selectedMethod]
        self.gameFileNameLabel.configure(text=combinedName)
        if (self.scriptIsModified(self.currentScript)):
            self.scriptBtn.menu.entryconfigure(index=3, state = Tkinter.NORMAL)
        else:    
            self.scriptBtn.menu.entryconfigure(index=3, state = Tkinter.DISABLED)
        
    def saveCurrScript(self):
        #call saveToXML on the current script, prompt for confirmation if the file already exists
        if (self.currentScript.name=="Patrol"): #don't mess up other scripts yet!!!
            filename = self.xmlFiles[self.currentScript.name]
            filename = datapath.fullname("scripts\\"+filename)
            self.currentScript.saveToXML(filename)
            self.modifiedScripts.remove(self.currentScript)
            self.updateFileNameLabel(self.currentScript)
    
    def newMethodDlg(self):
        dlg = Pmw.PromptDialog(self.root,title="New method",
            label_text="Enter name for new method:",
            entryfield_labelpos=Tkinter.N,defaultbutton=0,buttons=('OK','Cancel'))
        res = dlg.activate()
        if (res=='Cancel'):
            return
        newMethodName = dlg.component("entryfield").getvalue()
        scriptsPath = datapath.scriptspath()
        fname = tkFileDialog.asksaveasfilename(filetypes=[("all files", "*.py")],initialdir=scriptsPath)
        if (fname==""):
            return
        self.currentScript.methods[newMethodName] = ""
        fname = "scripts\\"+fname[len(scriptsPath):]
        self.currentScript.fileNames[newMethodName] = fname
        self.currentScript.saveMethod(newMethodName)
        self.modifiedScripts.append(self.currentScript) 
        self.modifiedMethods.append([self.currentScript,newMethodName])
        #update UI:
        self.methodList.insert(Tkinter.END,newMethodName) #insert in the head?
        n = self.methodList.size()
        self.methodList.selectitem(n-1,setentry=1)
        self.updateFileNameLabel(self.currentScript, newMethodName)

    def saveMethod(self):
        if (self.currentScript.name=="Patrol"): #don't mess up other scripts yet!!!
            selection = self.methodList.getcurselection()
            if (len(selection)==0):
                #show message!
                return
            self.currentMethod = selection[0]
            data = self.methodText.get()
            self.currentScript.methods[self.currentMethod] = data
            self.currentScript.saveMethod(self.currentMethod)
        
    def addInheritedDlg(self):
        scriptsPath = datapath.scriptspath()
        inheritFrom = tkFileDialog.askopenfilename(filetypes=[("all files", "*.xml")],initialdir=scriptsPath)
        if (os.path.exists(inheritFrom)):
            self.modifiedScripts.append(self.currentScript) 
            self.updateFileNameLabel(self.currentScript)
            scr = xmlscript.scriptFromFile(inheritFrom)
            self.currentScript.inherited.append(scr)
            self.resetCanvas()
            self.populateCanvas()
        
    def addScript(self, aScript, key=None, filename=None):
        """Add script to all internal lists and dictionaries."""
        if (key==None):
            self.scripts[aScript.name] = aScript
        else:
            self.scripts[key] = aScript
        if (filename==None):
            filename = aScript.name.lower()+".xml"
            fullname = datapath.fullname("scripts\\"+filename,create=1)
            aScript.saveToXML(fullname) #writing this will prevent xml parser from crashing
        self.xmlFiles[aScript.name] = filename
        self.scriptList.insert(Tkinter.END, aScript.name)
        self.scriptNames.append(aScript.name)
        self.allScripts.append(aScript)

    def createCanvasItem(self,canvas,scriptName,depth,row):
        frm = Tkinter.Frame(canvas,relief=Tkinter.SUNKEN,borderwidth=1)
        Tkinter.Button(frm,text=scriptName,command=lambda p = scriptName: self.selectFromCanvas(p) ).pack()
        x0 = self.posOnCanvas(depth,row)[0]
        y0 = self.posOnCanvas(depth,row)[1]
        canvas.create_window(x0,y0,window=frm,anchor=Tkinter.CENTER)
        self.labelPositions[scriptName] = (x0,y0)
    
    def addCanvasItem(self, aScript, depth, i):
        canvas = self.canvas.component("canvas")
        self.canvasLayers[depth].append(aScript.name)
        i = len(self.canvasLayers[depth])-1
        self.createCanvasItem(canvas,aScript.name,0,i)
    
    def newScriptDlg(self):
        dlg = Pmw.PromptDialog(self.root,title="New script",
            label_text="Enter name for new script:",
            entryfield_labelpos=Tkinter.N,defaultbutton=0,buttons=('OK','Cancel'))
        res = dlg.activate()
        if (res=='OK'):
            newScriptName = dlg.component("entryfield").getvalue()
            self.currentScript = xmlscript.xmlScript(name=newScriptName)
            self.addScript(self.currentScript)
            self.modifiedScripts.append(self.currentScript) 
            depth = 0
            self.addCanvasItem(self.currentScript,depth,len(self.canvasLayers[depth])-1)
            self.canvas.resizescrollregion()
            self.inheritedSelected(self.currentScript.name) #sets selection to the new item
                    
    def buildFileNamesFrame(self):
        self.fileNamesFrame = Tkinter.Frame(self.root, relief=Tkinter.SUNKEN, borderwidth=2)
        self.gameFileNameLabel = Tkinter.Label(self.fileNamesFrame, text="No game is loaded.", justify=Tkinter.RIGHT)
        self.gameFileNameLabel.pack(side=Tkinter.TOP, fill=Tkinter.X)
        self.fileNamesFrame.pack(side=Tkinter.TOP, fill=Tkinter.X)

    def buildXmlFilesList(self):
        self.middleFrame = Tkinter.Frame(self.root, relief=Tkinter.SUNKEN, borderwidth=2)

        self.scriptList = Pmw.ComboBox(self.middleFrame, label_text = 'Script:', labelpos = 'nw',
                selectioncommand = self.scriptSelected, scrolledlist_items = [], dropdown = 0)
        self.scriptList.pack(side = 'left', fill = 'both', expand = 1)

        self.inheritedList = Pmw.ComboBox(self.middleFrame, label_text = 'Inheirted:', labelpos = 'nw',
                selectioncommand = self.inheritedSelected, scrolledlist_items = [], dropdown = 0)
        self.inheritedList.pack(side = 'left', fill = 'both', expand = 1)

        self.methodList = Pmw.ComboBox(self.middleFrame, label_text = 'Method:', labelpos = 'nw',
                selectioncommand = self.methodSelected, scrolledlist_items = [], dropdown = 0)
        self.methodList.pack(side = 'left', fill = 'both', expand = 1)

        self.methodText = Pmw.HistoryText(self.middleFrame, text_wrap = 'none', text_width = 80, text_height = 20)
        self.methodText.pack(fill = 'both', expand = 1)
        self.methodText.component('text').focus()        

        self.middleFrame.pack(side=Tkinter.TOP, fill = Tkinter.X)

    def buildCanvas(self):
        self.canvas = Pmw.ScrolledCanvas(self.root,borderframe = 1,canvasmargin=10) #Tkinter.Canvas(self.root,height=300)
        self.canvas.pack(side=Tkinter.TOP, fill=Tkinter.BOTH, expand=Tkinter.YES)

    def methodSelected(self,selected):
        self.currentMethod = selected #make sure it has no "." in it!
        self.methodText.settext(self.currentScript.findMethod(selected))
        self.updateFileNameLabel(self.currentScript,selected)

    def collectInheritedMethods(self, aScript, methods):
        for s in aScript.inherited:
            dotmethods = s.methods.keys()
            for m in dotmethods:
                methods.append(s.name+"."+m)
            self.collectInheritedMethods(s,methods)
            
    def collectInheritedScripts(self, aScript, inhdscripts):
        for s in aScript.inherited:
            inhdscripts.append(s.name)
            self.collectInheritedScripts(s,inhdscripts)

    def scriptSelected(self,selected):
        for s in self.scripts.keys():
            aScript = self.scripts[s]
            if (aScript.name==selected):
                scriptFileName = self.xmlFiles[selected]
                self.currentScript = aScript
                self.scriptBtn.menu.entryconfigure(index=2, state = Tkinter.NORMAL)
                methods = self.currentScript.methods.keys()
                self.collectInheritedMethods(self.currentScript, methods)
                self.methodList.setlist(methods)
                self.methodText.settext("")
                inscripts = []
                self.collectInheritedScripts(aScript, inscripts)
                self.inheritedList.setlist(inscripts)
                self.updateFileNameLabel(self.currentScript)

    def inheritedSelected(self,selected):
        i = 1
        found = 0
        for n in self.scriptNames:
            print "name ",i,":",n
            if (n==selected):
                found = 1
                break
            i += 1
        if (found==0): i = 0
        self.scriptList.selectitem(i,setentry=1)
        self.scriptSelected(selected)
    
    def posOnCanvas(self,layerId,indexInLayer):
        return (50+layerId*200,40+40*indexInLayer)
    
    def populateCanvas(self):
        #build layers
        canvas = self.canvas.component("canvas")
        self.labelPositions = {}
        self.canvasLayers = {}
        for s in self.scripts.keys():
            depth = self.scripts[s].getInheritanceDepth()
            if (not self.canvasLayers.has_key(depth)):
                self.canvasLayers[depth] = []
            self.canvasLayers[depth].append(self.scripts[s].name)
        #draw layers:
        for d in self.canvasLayers.keys():
            layer = self.canvasLayers[d]
            i = 0
            for s in layer:
                self.createCanvasItem(canvas,s,d,i)
                i+=1
        #draw inheirtance
        for s in self.allScripts:
            for inh in s.inherited:
                x0,y0 = self.labelPositions[s.name]
                x1,y1 = self.labelPositions[inh.name]
                self.canvas.create_line(x0,y0,x1,y1)
        self.canvas.resizescrollregion()
    
    def selectFromCanvas(self, name):
        self.inheritedSelected(name)

    def openGame(self, gamefile=None):
        if (gamefile==None):
            self.gamefile = tkFileDialog.askopenfilename(filetypes=[("all files", "*.xml")],initialdir="..\\games")
            if (not os.path.exists(self.gamefile)):
                print "path does not exist: ",self.gamefile
                self.gamefile = None
                return
        else:
            self.gamefile = gamefile
        self.init()
        self.resetui()
        #print self.gamefile
        self.gameFileNameLabel.configure(text=self.gamefile)
        datapath.gameroot = os.path.splitext(self.gamefile)[0]+"\\" 
        #print datapath.gameroot
        self.startPath = datapath.gameroot + "\\scripts\\"
        #print self.startPath
        self.allFiles = os.listdir(self.startPath)
        #print self.allFiles
        self.collectXmlFiles()
        self.populateCanvas()
        #enable menu items
        self.gameBtn.menu.entryconfigure(index=2, state = Tkinter.NORMAL)
        self.scriptBtn.menu.entryconfigure(index=2, state = Tkinter.DISABLED)
        self.scriptBtn['state'] = Tkinter.NORMAL
        self.methodBtn['state'] = Tkinter.NORMAL

    def collectXmlFiles(self):
        #add Game Script:
        self.allScripts = []
        aScript = xmlscript.scriptFromFile(self.gamefile)
        self.scripts[self.gamefile] = aScript
        self.xmlFiles[aScript.name] = self.gamefile
        self.scriptList.insert(Tkinter.END, aScript.name)
        self.allScripts.append(aScript)
        
        for f in self.allFiles:
            parts = os.path.splitext(f)
            if (len(parts)>0) and (parts[1]==".xml"):
                sf = "scripts\\"+f
                aScript = xmlscript.scriptFromFile(datapath.fullname(sf))
                self.addScript(aScript, key=parts[0], filename=f)

if (__name__=="__main__"):
    e = ScriptEditor("..\\")
    #e.run(0)
