"""An utility module for loading animation from animlist xml files."""

import animation
import xml.dom.minidom
import xmlutils
import datapath
import actorvisual

def loadAnimlistFromXML(filename, animatedobj, canvas): 
    """An utility function that parses xml description of images that visual 
    has in its dictionary, parses and loads animation with names, frames 
    and durations."""
    file = open(filename)
    data = file.read()
    file.close()
    DOM = xml.dom.minidom.parseString(data)
    
    #print "animatedobj.visual=",animatedobj.visual
    
    #load image dictionary
    images = DOM.getElementsByTagName("IMAGE")
    imgDict = {}
    for im in images:
        name = None
        fname = None
        for childNode in im.childNodes:
            if (childNode.nodeType==xml.dom.minidom.Node.ELEMENT_NODE):
                if (childNode.tagName=="NAME"):
                    name = xmlutils.extractData(childNode)
                if (childNode.tagName=="FILE"):
                    fname = xmlutils.extractData(childNode)
                    fname = datapath.fullname(fname)
        if (name!=None) and (fname!=None):
            #print name, fname
            imgDict[name] = fname
    #done loading image dictionary

    animatedobj.setVisual(actorvisual.ActorVisual(canvas, imgDict, animatedobj.name))
    
    #load animations
    animatedobj.animPlayer.anims = {}
    anims = DOM.getElementsByTagName("ANIMATION")
    #print "(total number of animations) len(anims) = ", len(anims)
    for a in anims:
        animname = None
        frames = []
        for animfield in a.childNodes:
            if (animfield.nodeType==xml.dom.minidom.Node.ELEMENT_NODE):
                if (animfield.tagName=="NAME"):
                    animname = xmlutils.extractData(animfield)
                    #print "loading anim name= ", animname
                if (animfield.tagName=="FRAME"):
                    #print "frame found:"
                    num = -1
                    duration = -1
                    imgname = None
                    for frm in animfield.childNodes:
                        if (frm.nodeType==xml.dom.minidom.Node.ELEMENT_NODE):
                            if (frm.tagName=="NUM"):
                                num = xmlutils.extractData(frm)
                            if (frm.tagName=="DURATION"):
                                duration = xmlutils.extractData(frm)
                                #print "duration",int(duration)
                            if (frm.tagName=="IMGID"):
                                imgname = xmlutils.extractData(frm)
                    frames.append([int(duration),imgname])
                    #print num,duration,imgname
        animatedobj.animPlayer.anims[animname]=animation.Animation(name=animname,images=frames,visual=animatedobj.visual)
    #end of loading animations
    
    DOM.unlink() #call this to clear DOM memory!

    