"""Exports xmlScript class used by ScriptableAgent. 

xmlScript extends abstract Script with ability to load code from .py files
after reading list of methods and inherited scripts from master .xml file.
"""

from osm import script
import xml.dom.minidom
import string
import os
import datapath
import xmlutils

class xmlScript(script.Script):
    """Can load individual methods from .py files after reading master 
    xml file with the list of all methods and names of inherited scripts.
    """
    
    def __init__(self, owner=None, name=""):
        script.Script.__init__(self, owner, name)
        self.fileNames = {}

    def loadMethodFromFile(self, name, filename):
        """Simple loader for a single method from a text file. 
        This function is called by loadFromXML."""
        f = open(datapath.fullname(filename))
        self.methods[name] = f.read()
        f.close()
        self.fileNames[name] = filename

    def getInheritanceDepth(self):
        """Helper function for script browser. Not used in the game code."""
        if (len(self.inherited)==0):
            return 0
        depths = []
        for inscript in self.inherited:
            depths.append(1+inscript.getInheritanceDepth())
        maxD = 0
        for d in depths:
            if (d > maxD):
                maxD = d
        return maxD
        
    def saveMethod(self,name):
        print "saveMethod", name
        f = open(datapath.fullname(self.fileNames[name],create=1),'w')
        f.write(self.methods[name])
        f.close()
                
    def saveToXML(self,filename):
        DOM = xml.dom.minidom.Document()
        scriptElem = DOM.createElement("SCRIPT")
        scriptNameElem = DOM.createElement("SCRIPTNAME")
        scriptElem.appendChild(scriptNameElem)
        textElem = DOM.createTextNode(self.name)
        scriptNameElem.appendChild(textElem)
        DOM.appendChild(scriptElem)
        #write inherited scripts
        for inh in self.inherited:
            usesElem = DOM.createElement("USES")
            scriptElem.appendChild(usesElem)
            inhFileName = "scripts\\"+inh.name.lower()+".xml"
            textElem = DOM.createTextNode(inhFileName)
            usesElem.appendChild(textElem)
        for m in self.methods.keys():
            fname = self.fileNames[m]
            methodElem = DOM.createElement("METHOD")
            scriptElem.appendChild(methodElem)
            methodNameElem = DOM.createElement("NAME")
            methodNameText = DOM.createTextNode(m)
            methodNameElem.appendChild(methodNameText)
            methodElem.appendChild(methodNameElem)
            methodBodyElem = DOM.createElement("BODY")
            methodBodyText = DOM.createTextNode(fname)
            methodBodyElem.appendChild(methodBodyText)
            methodElem.appendChild(methodBodyElem)
            self.saveMethod(m)
        #convert to ascii and save:
        data = DOM.toxml()
        DOM.unlink()        
        file = open(filename,'w')
        file.write(data)
        file.close()
        
    def loadFromXML(self, filename):
        """Reads master xml file, then loads referenced files for methods 
        and inehrited scripts."""
        self.xmlfilename = filename
        file = open(filename)
        data = file.read()
        file.close()
        DOM = xml.dom.minidom.parseString(data)
        #extract script name:
        scriptname = DOM.getElementsByTagName("SCRIPTNAME")
        for sn in scriptname:
            self.name = xmlutils.extractData(sn)
        #load inherited:
        self.inherited = []
        used = DOM.getElementsByTagName("USES")
        for s in used:
            fname = xmlutils.extractData(s)
            #print "extracted fname: ",fname
            fname = datapath.fullname(fname)
            #print "fname for inherited script: ",fname
            #scr = Script().loadFromXML(fname)
            scr = scriptFromFile(fname)
            self.inherited.append(scr)
        #load methods:
        xmlmethods = DOM.getElementsByTagName("METHOD")
        for xmlmethod in xmlmethods:
            name = None
            body = None
            for childNode in xmlmethod.childNodes:
                if (childNode.nodeType==xml.dom.minidom.Node.ELEMENT_NODE):
                    if (childNode.tagName=="NAME"):
                        name = xmlutils.extractData(childNode)
                    if (childNode.tagName=="BODY"):
                        fname = xmlutils.extractData(childNode)
                        self.loadMethodFromFile(name, fname)
            if (name!=None) and (body!=None):
                self.methods[name]=body
        DOM.unlink()
        return self

    def handleExceptionInMethod(self,name,method):
        """Points at the file and line where error/exception occurs in 
        the script nur by exec function."""
        import sys,traceback
        print "ERROR:"
        traceback.print_exc(file=sys.stdout)
        print ">>can't execute method ", name
        print ">>declared in: ", self.xmlfilename
        print ">>imported from: ", self.fileNames[name]
        print sys.exc_type, sys.exc_value

# a handy function:
def scriptFromFile(filename):
    """A shortcut for creating script from a filename."""
    s = xmlScript()
    s.loadFromXML(filename)
    return s

